#! /usr/bin/env node
"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

var _os = require("os");

var _child_process = require("child_process");

var _libLang = require("@verekia/lib-lang");

var _safe = _interopRequireDefault(require("colors/safe"));

var _checkSetup = require("@sharyn/check-setup");

var _shared = require("./shared");

var _commands = require("./commands");

// flow-disable-next-line
var hasDocker = (0, _checkSetup.hasFile)('docker-compose.yml');
var hasHeroku = (0, _checkSetup.hasFile)('Procfile');
var hasSeeds = (0, _checkSetup.hasFile)('/src/_db/seeds');

var getDbTestProcessId = function getDbTestProcessId() {
  var dbTestContainerName = 'db-test';
  var result = (0, _child_process.execSync)("docker ps -q --filter=\"name=".concat(dbTestContainerName, "\"")).toString();
  var ids = result.split(_os.EOL).filter(function (x) {
    return x;
  });

  if (ids.length > 1) {
    throw Error("Multiple running processes found for ".concat(dbTestContainerName));
  }

  return ids[0];
};

var mySpawn = function mySpawn(cmd) {
  // eslint-disable-next-line no-console
  console.log("".concat(_safe.default.magenta("[sharyn/cli]"), " ").concat(_safe.default.gray(cmd)).replace(/\.\/node_modules\/\.bin\//g, ''));
  return (0, _child_process.spawn)(cmd, {
    shell: true,
    stdio: 'inherit'
  });
};

var sequence = function sequence(arr) {
  return arr.join(' && ');
};

var taskName = process.argv[2];
(0, _libLang.swit)(taskName, [['dev', function () {
  var firstCommands = [];
  hasDocker && firstCommands.push(_commands.DOCKER_UP);
  _shared.knexConfigPath && firstCommands.push(_commands.DOCKER_WAIT_PG, _commands.dbMigr);
  _shared.knexConfigPath && hasSeeds && firstCommands.push(_commands.dbSeed);
  firstCommands.push(_commands.rmBundle);
  mySpawn(sequence(firstCommands)).on('close', function (code) {
    if (code === 0) {
      mySpawn(_commands.serverWatch);
      mySpawn(_commands.clientWatch);
    }
  });
}], ['dev-ssr-only', function () {
  var commands = [];
  hasDocker && commands.push(_commands.DOCKER_UP);
  _shared.knexConfigPath && commands.push(_commands.DOCKER_WAIT_PG, _commands.dbMigr);
  _shared.knexConfigPath && hasSeeds && commands.push(_commands.dbSeed);
  commands.push(_commands.serverWatchSsrOnly);
  mySpawn(sequence(commands));
}], ['dev-no-ssr', function () {
  var firstCommands = [];
  hasDocker && firstCommands.push(_commands.DOCKER_UP);
  _shared.knexConfigPath && firstCommands.push(_commands.DOCKER_WAIT_PG, _commands.dbMigr);
  _shared.knexConfigPath && hasSeeds && firstCommands.push(_commands.dbSeed);
  firstCommands.push(_commands.rmBundle);
  mySpawn(sequence(firstCommands)).on('close', function (code) {
    if (code === 0) {
      mySpawn(_commands.serverWatchNoSsr);
      mySpawn(_commands.clientWatch);
    }
  });
}], ['local-prod', function () {
  var commands = [];
  hasDocker && commands.push(_commands.DOCKER_UP);
  _shared.knexConfigPath && commands.push(_commands.DOCKER_WAIT_PG, _commands.dbMigr);
  _shared.knexConfigPath && hasSeeds && commands.push(_commands.dbSeed);
  commands.push(_commands.rmLibAndBundle, _commands.clientBuild, _commands.babel);
  hasHeroku ? commands.push(_commands.herokuLocalProd) : commands.push(_commands.nodeLocalProd);
  mySpawn(sequence(commands));
}], ['build-prod', function () {
  return mySpawn(sequence([_commands.rmLibAndBundle, _commands.clientBuild, _commands.babel]));
}], ['lint', function () {
  return mySpawn(sequence([_commands.lint, _commands.typecheck]));
}], ['test', function () {
  var commands = [];
  var testDbId = getDbTestProcessId();
  hasDocker && testDbId && commands.push((0, _commands.dockerDownTest)(testDbId));
  hasDocker && commands.push(_commands.DOCKER_UP_TEST);
  _shared.knexConfigPath && commands.push(_commands.DOCKER_WAIT_PG_TEST, _commands.dbMigrTest);
  commands.push(_commands.rmBundle, _commands.testUnit, _commands.clientBuild, _commands.testE2E);
  mySpawn(sequence(commands));
}], ['lint-test', function () {
  var commands = [_commands.lint, _commands.typecheck];
  var testDbId = getDbTestProcessId();
  hasDocker && testDbId && commands.push((0, _commands.dockerDownTest)(testDbId));
  hasDocker && commands.push(_commands.DOCKER_UP_TEST);
  _shared.knexConfigPath && commands.push(_commands.DOCKER_WAIT_PG_TEST, _commands.dbMigrTest);
  commands.push(_commands.rmBundle, _commands.testUnit, _commands.clientBuild, _commands.testE2E);
  mySpawn(sequence(commands));
}], ['migrate-db', function () {
  return mySpawn(_commands.dbMigr);
}], ['stats', function () {
  return mySpawn(_commands.stats);
}]], function () {
  // eslint-disable-next-line no-console
  console.error("".concat(taskName, " is not a valid @sharyn/cli command."));
  process.exit(1);
});